/***************************************************************************
 *   Copyright (C) 2006 by André Lindhjem <belgarat@sdf.lonestar.org>,     *
 *                         Kjetil Holien <kjetil.holien@gmail.com>,        *
 *                         Terje Risa <terje.risa@gmail.com> &             *
 *                         Øyvind Nerbråten <oyvind@nerbraten.com>         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************
 *
 * \file daisyplayer.c
 * \author André Lindhjem, Kjetil Holien, Terje Risa & Øyvind Nerbråten
 * \date 15.02.2006
 * 
 * Needed to link: -lxml2 -lao -lmad -lfaad
 */

#include <signal.h>
#include <string.h>
#include <unistd.h>
#include <ncurses.h>	/** add -lncurses to build! */

#include "audio.h" //TODO: this will be removed when a proper playback interface for the engine is in place.
#include "control.h" //TODO: Make a new interface for the engine, this is for internal use..


/**
 * Function declarations
 */
static
void sigproc ();		/** Listens for CTRL-C and does it's best to free resources in a Good Way */

void callback_playback_done (void *no);	/** callback function. this will be called once the audio engine is done with a segment */

void event_pause ();	/** event functions */
void event_play ();
void event_stop ();
void event_prev_chap ();
void event_next_chap ();
void event_prev_para ();
void event_next_para ();

/**
 * Global vars
 */
static int playing = 1;
static bool is_playing = false;
unsigned long int duration_ms = 0;	/** the duration of the segment we are playing */


/**
 * Main function
 */
int main (int argc, char *argv[]) {
	int ret = 0;
	
	// intialising the mp3 and xmlparsing librarys
	parseInit();
//	callback_playback_done(NULL);
	start_audio (&callback_playback_done);
	
	//  adding signal handling. this should catch ctrl-c and free the resources because this is a Good Thing 
	(void) signal(SIGINT, sigproc);
	
	
	// open a ncc/ncx file and start parsing
	if (argc == 2)
	{
		printf ("\nReading from %s\n",argv[1]);
		ret = parse(argv[1]);
	}
	else 
	{
		printf ("\nPlease enter full path to the Daisy book (../ncc.html | ../*.ncx)\n");
		char string[500];
		(void) scanf("%s", string);
		if(string != NULL)
		{
			printf ("\nReading from %s\n",string);
			ret = parse(string);
		}			
	}

	
	// We can now play!
	if (ret == 1)
	{
		printf("\n\n");
//		int playing = 1;
		int keypress = 0;	/** user input */

		playing = seek (2, __ncx);	// seeking to the first passage

		initscr ();	/* start the curses mode */
		noecho ();	/* don't echo to screen */
		keypad (stdscr, TRUE);		/** enable keypad keys */
		scrollok(stdscr, TRUE);
		idlok(stdscr, TRUE);
		
		int row, col;
		row = 1, col = 10;
		mvprintw (row, col, "Welcome to daisyplayer!");
		row = 3, col = 5;
		mvprintw (row, col, "Commands:\n");
		col += 5;
		mvprintw (row+1, col, "p = play\n");
		mvprintw (row+2, col, "s = stop\n");
		mvprintw (row+3, col, "space = pause/play\n");
		mvprintw (row+4, col, "left arrow = prev passage\n");
		mvprintw (row+5, col, "right arrow = next passage\n");
		mvprintw (row+6, col, "up arrow = prev chapter\n");
		mvprintw (row+7, col, "down arrow = next chapter\n");
		mvprintw (row+8, col, "q = quit\n\n");
		
		while (keypress != 'q')		/** q quits */
		{
			keypress = wgetch (stdscr);
	
			switch (keypress)
			{
				case KEY_UP:
					event_prev_chap ();
					break;
				case KEY_DOWN:
					event_next_chap ();
					break;
				case KEY_LEFT:
					event_prev_para ();
					break;
				case KEY_RIGHT:
					event_next_para ();
					break;
				case ' ':
					event_pause ();
					break;
				case 'p':
					event_play ();
					break;
				case 's':
					event_stop ();
					break;				
				default:
					printw ("\nunknown key: %d\n", keypress);
					break;
			}
		}

//		while (playing == 1)
//		{
//			if (__nodePos->textPassage != NULL)
//				printf("%s\n", __nodePos->textPassage);
//			if (__nodePos->audiofilename != NULL)
//			{
//				char tmp[STRLEN];
//				snprintf (tmp, sizeof (tmp), "%s%s", __path, __nodePos->audiofilename);
//				
//				duration_ms = audio_play(tmp, __nodePos->audioStartPos, __nodePos->audioStopPos);
//				/**
//				 * NOTE: New functions that (apparently seems to) work (kindof) are:
//				 * Tell André if they don't do what you want.
//				 * audio_stop ();
//				 * audio_pause ();
//				 * audio_block ();
//				 */
//			}
//			// if we haven't got any audiosource the text must be delayed for an appropriate time, this
//			// is done by sleeping in the length of the string divided by 40.
//			else
//			{
//				if (__nodePos->textPassage != NULL)	/** make sure __nodePos->textPassage is not NULL */
//				duration_ms = ((unsigned long int) strlen (__nodePos->textPassage)/40);
//			}
//			(void) usleep((duration_ms * 1000));	/** convert to us and sleep */
//			playing = seek (2, __ncx);	// seek to next passage
//		}
		printw ("\n\nEnd of book\n");
	}
	endwin ();
	parseTerminate ();	/** tell parser to terminate and clean up */
	stop_audio ();
	return 0;
}


/**
 * Listens for CTRL-C and does it's best to free resources in a Good Way
 */
static void sigproc()
{
	(void) signal(SIGINT, sigproc);	/** reset the signal */
	printw ("Recieved ctrl-c. Cleaning up and exiting.\n");
	endwin ();
	parseTerminate();		/** tell parser to terminate and clean up		*/
	stop_audio ();
	exit (0);
}


/**
 * This is called by the audio engine when a playback is done.
 * We'll go to the next passage and play audio again
 */
void callback_playback_done (void *no)
{
	if (is_playing != false)	/** if this is false, we are instructed to stop */
	{
		is_playing = false;	/** finished playing */
		playing = seek (2, __ncx);	// seek to next passage
		event_play ();
	}
}


/**
 * Event functions!
 */
void event_pause ()
{
	audio_pause();
}


void event_play ()
{
	is_playing = true;

	if (__nodePos->textPassage != NULL)
//		printf("%s\n", __nodePos->textPassage);
		printw("%s\n", __nodePos->textPassage);
		refresh ();
	if (__nodePos->audiofilename != NULL)
	{
		char tmp[STRLEN];
		snprintf (tmp, sizeof (tmp), "%s%s", __path, __nodePos->audiofilename);
		duration_ms = audio_play(tmp, __nodePos->audioStartPos, __nodePos->audioStopPos);
		/**
		 * NOTE: New functions that (apparently seems to) work (kindof) are:
		 * Tell André if they don't do what you want.
		 * audio_stop ();
		 * audio_pause ();
		 * audio_block ();
		 */
	}
	// if we haven't got any audiosource the text must be delayed for an appropriate time, this
	// is done by sleeping in the length of the string divided by 40.
	else
	{
		if (__nodePos->textPassage != NULL)	/** make sure __nodePos->textPassage is not NULL */
		duration_ms = ((unsigned long int) strlen (__nodePos->textPassage)/40);
	}

//	(void) usleep((duration_ms * 1000));	/** convert to us and sleep */
}


void event_stop ()
{
	audio_stop();	/** stopping the audio engine */
	is_playing = false;	/** stop our parsing too */
}


void event_prev_chap ()
{
	//printw ("event: prev chap\n");
	event_stop();
	usleep (1*200000);
	seek (-1, __ncx);
	event_play();
}


void event_next_chap ()
{
	//printw ("event: next chap\n");
	event_stop();
	usleep (1*200000);
	seek (1, __ncx);
	event_play();
}


void event_prev_para ()
{
	//printw ("event: prev para\n");
	event_stop();
	usleep (1*200000);
	seek (-2, __ncx);
	event_play();
}


void event_next_para ()
{
	//printw ("event: next para\n");
	event_stop();
	usleep (1*200000);
	seek (2, __ncx);
	event_play();
}
