/***************************************************************************
 *   Copyright (C) 2006 by André Lindhjem <belgarat@sdf.lonestar.org>,     *
 *                         Kjetil Holien <kjetil.holien@gmail.com>,        *
 *                         Terje Risa <terje.risa@gmail.com> &             *
 *                         Øyvind Nerbråten <oyvind@nerbraten.com>         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************
 *
 * \file parsetime.c
 * \author André Lindhjem, Kjetil Holien, Terje Risa & Øyvind Nerbråten
 * \date 15.02.2006
 * 
 * This library parses valid DAISY time formats and constructs mad_timer_t
 */

#ifndef PARSETIME_H_
#define PARSETIME_H_


/**
 * enum that specifies Full-clock, Partial-clock, Timecount
 */
enum pt_type {
	PT_INVALID_FORMAT	= -1,
	PT_FULL_CLOCK		= 0,
	PT_PARTIAL_CLOCK	= 1,
	PT_TIMECOUNT		= 2,
};


enum pt_unit {
	PT_NO_UNIT 			= 0,
	PT_MILLISECONDS		= 1,
	PT_SECONDS 			= 2,
	PT_MINUTES 			= 3,
	PT_HOURS			= 4,
};


/**
 * Translates valid DAISY time formats to seconds and fractions
 * Ref: sec. 7.7, p41 in ANSI/NISO Z39.86.2005
 */
mad_timer_t parse_time_mad_time_t (char *expr);


/**
 * Translates valid DAISY time formats to milliseconds
 * Ref: sec. 7.7, p41 in ANSI/NISO Z39.86.2005
 */
unsigned long int parse_time_milliseconds (char *expr);


/**
 * Returns the duration in milliseconds
 */
unsigned long int parse_time_get_dur (char *start, char *stop);


#endif /*PARSETIME_H_*/
